@@ -1,9 +1,9 @@
-#' Non-Linear Regression DIF statistic.
+#' DIF statistics based on non-linear regression model.
 #'
 #' @aliases NLR
 #'
-#' @description Performs DIF detection procedure based on Non-Linear Regression and either
-#' likelihood ratio test or F-test of submodel.
+#' @description Calculates either DIF likelihood ratio or F statistics for dichotomous data
+#' based on non-linear regression model (generalized logistic regression model).
 #'
 #' @param Data numeric: either binary data matrix only, or the binary data matrix plus the vector of group .
 #' See \strong{Details}.
@@ -18,7 +18,7 @@
 #' default, all items are considered as anchors. Argument is ignored if \code{match} is not \code{"zscore"} or \code{"score"}.
 #' See \strong{Details}.
 #' @param type character: type of DIF to be tested. Possible values are \code{"both"} (default), \code{"udif"},
-#' \code{"nudif"}, \code{"all"}, or combination of parameters 'a', 'b', 'c' and 'd'. See \strong{Details}.
+#' \code{"nudif"}, \code{"all"}, or combination of parameters \code{"a"}, \code{"b"}, \code{"c"} and \code{"d"}. See \strong{Details}.
 #' @param p.adjust.method character: method for multiple comparison correction. See \strong{Details}.
 #' @param start numeric: matrix with n rows (where n is the number of items) and 8 columns containing initial
 #' item parameters estimates. See \strong{Details}.
@@ -38,9 +38,9 @@
 #' DIF detection procedure based on Non-Linear Regression is the extension
 #' of Logistic Regression procedure (Swaminathan and Rogers, 1990).
 #'
-#' The \code{Data} is a matrix whose rows represents examinee scored answers
-#' ("1" correct, "0" incorrect) and columns correspond to the items.
-#' The \code{group} must be a vector of the same length as \code{nrow(data)}.
+#' The \code{Data} is a matrix which rows represents examinee scored answers
+#' (1 - correct, 0 - incorrect) and columns correspond to the items.
+#' The \code{group} must be a vector of the same length as \code{nrow(Data)}.
 #'
 #' The unconstrained form of 4PL generalized logistic regression model for probability of
 #' correct answer (i.e., y = 1) is
@@ -129,10 +129,10 @@
 #'   1 means convergence issue in m1 model.}
 #' }
 #' @author
-#' Adela Drabinova \cr
+#' Adela Hladka (nee Drabinova) \cr
 #' Institute of Computer Science, The Czech Academy of Sciences \cr
 #' Faculty of Mathematics and Physics, Charles University \cr
-#' drabinova@cs.cas.cz \cr
+#' hladka@cs.cas.cz \cr
 #'
 #' Patricia Martinkova \cr
 #' Institute of Computer Science, The Czech Academy of Sciences \cr
@@ -183,12 +183,9 @@
 #' # using maximum likelihood estimation method
 #' NLR(Data, group, model = "3PLcg", method = "likelihood")
 #' }
+#'
 #' @keywords DIF
 #' @export
-#' @importFrom stats logLik
-#' @importFrom msm deltamethod
-
-
 NLR <- function(Data, group, model, constraints = NULL, type = "both",
                 method = "nls", match = "zscore", anchor = 1:ncol(Data),
                 start, p.adjust.method = "none", test = "LR", alpha = 0.05,
@@ -209,8 +206,8 @@ NLR <- function(Data, group, model, constraints = NULL, type = "both",
     }
   }
 
-  m <- ncol(Data)
-  n <- nrow(Data)
+  m <- dim(Data)[2]
+  n <- dim(Data)[1]
 
   if (length(model) == 1){
     model <- rep(model, m)
@@ -281,7 +278,7 @@ NLR <- function(Data, group, model, constraints = NULL, type = "both",
     startBo0[which(cfM0)] <- 1; startBo1[which(cfM1)] <- 1
     for (i in 1:nrBo){
       if (conv.fail > 0){
-        samp <- sample(1:nrow(Data), size = nrow(Data), replace = T)
+        samp <- sample(1:dim(Data)[1], size = dim(Data)[1], replace = T)
         startalt <- startNLR(Data[samp, ], group[samp], model, match = x,
                              parameterization = parameterization)
         if (sum(cfM0) > 0){
